@@ -5,48 +5,59 @@
 import os
 import re
 from collections import namedtuple
-from functools import partial
+from contextlib import suppress
+from functools import lru_cache, partial
 from qt.core import (
     QAction, QApplication, QClipboard, QColor, QDialog, QEasingCurve, QIcon,
-    QKeySequence, QMenu, QMimeData, QPainter, QPen, QPixmap, QSplitter,
-    QPropertyAnimation, QRect, QSize, QSizePolicy, Qt, QUrl, QWidget, pyqtProperty,
-    QTimer, pyqtSignal
+    QKeySequence, QMenu, QMimeData, QPainter, QPalette, QPen, QPixmap,
+    QPropertyAnimation, QRect, QSize, QSizePolicy, QSplitter, Qt, QTimer, QUrl, QWidget,
+    pyqtProperty, pyqtSignal,
 )
 
 from calibre import fit_image, sanitize_file_name
 from calibre.constants import config_dir, iswindows
+from calibre.db.constants import DATA_DIR_NAME, DATA_FILE_PATTERN, RESOURCE_URL_SCHEME
 from calibre.ebooks import BOOK_EXTENSIONS
 from calibre.ebooks.metadata.book.base import Metadata, field_metadata
 from calibre.ebooks.metadata.book.render import mi_to_html
 from calibre.ebooks.metadata.search_internet import (
     all_author_searches, all_book_searches, name_for, url_for_author_search,
-    url_for_book_search
+    url_for_book_search,
 )
 from calibre.gui2 import (
     NO_URL_FORMATTING, choose_save_file, config, default_author_link, gprefs,
-    pixmap_to_data, rating_font, safe_open_url
+    pixmap_to_data, question_dialog, rating_font, safe_open_url,
 )
 from calibre.gui2.dialogs.confirm_delete import confirm, confirm as confirm_delete
 from calibre.gui2.dnd import (
-    dnd_get_files, dnd_get_image, dnd_has_extension, dnd_has_image, image_extensions
+    dnd_get_files, dnd_get_image, dnd_has_extension, dnd_has_image, image_extensions,
 )
 from calibre.gui2.widgets2 import HTMLDisplay
+from calibre.startup import connect_lambda
 from calibre.utils.config import tweaks
 from calibre.utils.img import blend_image, image_from_x
 from calibre.utils.localization import is_rtl, langnames_to_langcodes
+from calibre.utils.resources import get_path as P
 from calibre.utils.serialize import json_loads
 from polyglot.binary import from_hex_bytes
 
 InternetSearch = namedtuple('InternetSearch', 'author where')
 
 
-def set_html(mi, html, text_browser):
+def db_for_mi(mi):
     from calibre.gui2.ui import get_gui
-    gui = get_gui()
+    lp = getattr(mi, 'external_library_path', None)
+    if lp:
+        return get_gui().library_broker.get_library(lp), True
+    return get_gui().current_db, False
+
+
+def set_html(mi, html, text_browser):
     book_id = getattr(mi, 'id', None)
     search_paths = []
-    if gui and book_id is not None:
-        path = gui.current_db.abspath(book_id, index_is_id=True)
+    db, _ = db_for_mi(mi)
+    if db and book_id is not None:
+        path = db.abspath(book_id, index_is_id=True)
         if path:
             search_paths = [path]
     text_browser.setSearchPaths(search_paths)
@@ -65,6 +76,16 @@ def css(reset=False):
     return css.ans
 
 
+def resolve_colors(css):
+    app = QApplication.instance()
+    col = app.palette().color(QPalette.ColorRole.PlaceholderText).name() if app.is_dark_theme else '#666'
+    return css.replace('palette(placeholder-text)', col)
+
+
+def resolved_css():
+    return resolve_colors(css())
+
+
 def copy_all(text_browser):
     mf = getattr(text_browser, 'details', text_browser)
     c = QApplication.clipboard()
@@ -74,22 +95,33 @@ def copy_all(text_browser):
     from html5_parser import parse
     from lxml import etree
     root = parse(html)
-    for x in ('table', 'tr', 'tbody'):
-        for tag in root.iterdescendants(x):
-            tag.tag = 'div'
-    for tag in root.iterdescendants('td'):
+    tables = tuple(root.iterdescendants('table'))
+    for tag in root.iterdescendants(('table', 'tr', 'tbody')):
+        tag.tag = 'div'
+    parent = root
+    is_vertical = getattr(text_browser, 'vertical', True)
+    if not is_vertical:
+        parent = tables[1]
+    for tag in parent.iterdescendants('td'):
+        for child in tag.iterdescendants('br'):
+            child.tag = 'span'
+            child.text = '\ue000'
         tt = etree.tostring(tag, method='text', encoding='unicode')
         tag.tag = 'span'
         for child in tuple(tag):
             tag.remove(child)
         tag.text = tt.strip()
+    if not is_vertical:
+        for tag in root.iterdescendants('td'):
+            tag.tag = 'div'
     for tag in root.iterdescendants('a'):
         tag.attrib.pop('href', None)
     from calibre.utils.html2text import html2text
     simplified_html = etree.tostring(root, encoding='unicode')
     txt = html2text(simplified_html, single_line_break=True).strip()
+    txt = txt.replace('\ue000', '\n\t')
     if iswindows:
-        txt = '\r\n'.join(txt.splitlines())
+        txt = os.linesep.join(txt.splitlines())
     # print(simplified_html)
     # print(txt)
     md.setText(txt)
@@ -137,6 +169,37 @@ def init_manage_action(ac, field, value):
     return ac
 
 
+def add_notes_context_menu_actions(menu, book_info, field, value):
+    from calibre.gui2.ui import get_gui
+    db = get_gui().current_db.new_api
+    if db.field_supports_notes(field):
+        item_id = db.get_item_id(field, value)
+        if item_id is not None:
+            val = escape_for_menu(value)
+            def edit_note():
+                gui = get_gui()
+                from calibre.gui2.dialogs.edit_category_notes import EditNoteDialog
+                d = EditNoteDialog(field, item_id, gui.current_db.new_api, parent=book_info)
+                if d.exec() == QDialog.DialogCode.Accepted:
+                    gui.do_field_item_value_changed()
+            ac = menu.addAction(_('Edit note for {}').format(val))
+            ac.triggered.connect(edit_note)
+            ac.setIcon(QIcon.ic('edit_input.png'))
+
+            def delete_note():
+                gui = get_gui()
+                if question_dialog(gui, _('Are you sure?'),
+                        _('Are you sure you want to delete the note for {} from this library? '
+                          'There is no undo.').format(val),
+                        skip_dialog_name='book_details_delete_note_context_menu'):
+                    db = gui.current_db.new_api
+                    db.set_notes_for(field, item_id, '')
+                    gui.do_field_item_value_changed()
+            ac = menu.addAction(_('Delete note for {}').format(escape_for_menu(value)))
+            ac.triggered.connect(delete_note)
+            ac.setIcon(QIcon.ic('trash.png'))
+
+
 def init_find_in_tag_browser(menu, ac, field, value):
     from calibre.gui2.ui import get_gui
     hidden_cats = get_gui().tags_view.model().hidden_categories
@@ -197,15 +260,23 @@ def init_find_in_grouped_search(menu, field, value, book_info):
                     '{}:"={}"'.format(g, value.replace('"', r'\"')), ''))
 
 
-def render_html(mi, vertical, widget, all_fields=False, render_data_func=None, pref_name='book_display_fields'):  # {{{
-    from calibre.gui2.ui import get_gui
+@lru_cache(maxsize=2)
+def comments_pat():
+    return re.compile(r'<!--.*?-->', re.DOTALL)
+
+
+def render_html(mi, vertical, widget, all_fields=False, render_data_func=None,
+                pref_name='book_display_fields',
+                pref_value=None):  # {{{
+    db, is_external = db_for_mi(mi)
+    show_links = not is_external
     func = render_data_func or partial(render_data,
-                   vertical_fields=get_gui().current_db.prefs.get('book_details_vertical_categories') or ())
+                   vertical_fields=db.prefs.get('book_details_vertical_categories') or ())
     try:
-        table, comment_fields = func(mi, all_fields=all_fields,
+        table, comment_fields = func(mi, all_fields=all_fields, show_links=show_links,
                 use_roman_numbers=config['use_roman_numerals_for_series_number'], pref_name=pref_name)
     except TypeError:
-        table, comment_fields = func(mi, all_fields=all_fields,
+        table, comment_fields = func(mi, all_fields=all_fields, show_links=show_links,
                 use_roman_numbers=config['use_roman_numerals_for_series_number'])
 
     def color_to_string(col):
@@ -227,6 +298,8 @@ def render_html(mi, vertical, widget, all_fields=False, render_data_func=None, p
     comments = ''
     if comment_fields:
         comments = '\n'.join('<div>%s</div>' % x for x in comment_fields)
+        # Comments cause issues with rendering in QTextBrowser
+        comments = comments_pat().sub('', comments)
     right_pane = comments
 
     if vertical:
@@ -238,9 +311,8 @@ def render_html(mi, vertical, widget, all_fields=False, render_data_func=None, p
     return ans
 
 
-def get_field_list(fm, use_defaults=False, pref_name='book_display_fields'):
-    from calibre.gui2.ui import get_gui
-    db = get_gui().current_db
+def get_field_list(fm, use_defaults=False, pref_name='book_display_fields', mi=None):
+    db, _ = db_for_mi(mi)
     if use_defaults:
         src = db.prefs.defaults
     else:
@@ -258,14 +330,29 @@ def get_field_list(fm, use_defaults=False, pref_name='book_display_fields'):
 
 
 def render_data(mi, use_roman_numbers=True, all_fields=False, pref_name='book_display_fields',
-                vertical_fields=()):
-    field_list = get_field_list(getattr(mi, 'field_metadata', field_metadata), pref_name=pref_name)
+                vertical_fields=(), show_links=True):
+    field_list = get_field_list(getattr(mi, 'field_metadata', field_metadata),
+                                pref_name=pref_name, mi=mi)
     field_list = [(x, all_fields or display) for x, display in field_list]
+    db, _ = db_for_mi(mi)
+    db = db.new_api
+    field_maps = {}
+
+    def item_id_if_has_note(field, item_val):
+        if db.field_supports_notes(field):
+            nmap = field_maps.get(field)
+            if nmap is None:
+                nmap = field_maps[field] = db.get_item_name_map(field)
+            item_id = nmap.get(item_val)
+            if item_id is not None:
+                if db.notes_for(field, item_id):
+                    return item_id
+
     return mi_to_html(
         mi, field_list=field_list, use_roman_numbers=use_roman_numbers, rtl=is_rtl(),
         rating_font=rating_font(), default_author_link=default_author_link(),
         comments_heading_pos=gprefs['book_details_comments_heading_pos'], for_qt=True,
-        vertical_fields=vertical_fields
+        vertical_fields=vertical_fields, show_links=show_links, item_id_if_has_note=item_id_if_has_note
     )
 
 # }}}
@@ -360,6 +447,7 @@ def add_item_specific_entries(menu, data, book_info, copy_menu, search_menu):
         init_find_in_tag_browser(search_menu, find_action, 'authors', author)
         init_find_in_grouped_search(search_menu, 'authors', author, book_info)
         menu.addAction(init_manage_action(book_info.manage_action, 'authors', author))
+        add_notes_context_menu_actions(menu, book_info, 'authors', author)
         if hasattr(book_info, 'search_internet'):
             search_menu.addSeparator()
             search_menu.sim = create_search_internet_menu(book_info.search_internet, author)
@@ -373,6 +461,12 @@ def add_item_specific_entries(menu, data, book_info, copy_menu, search_menu):
             ac.data = ('authors', author, book_id)
             ac.setText(_('Remove %s from this book') % escape_for_menu(author))
             menu.addAction(ac)
+        # See if we need to add a click associated link menu line for the author
+        link_map = get_gui().current_db.new_api.get_all_link_maps_for_book(data.get('book_id', -1))
+        link = link_map.get("authors", {}).get(author)
+        if link:
+            menu.addAction(QIcon.ic('external-link'), _('Open associated link'),
+                           lambda : book_info.link_clicked.emit(link))
     elif dt in ('path', 'devpath'):
         path = data['loc']
         ac = book_info.copy_link_action
@@ -381,6 +475,15 @@ def add_item_specific_entries(menu, data, book_info, copy_menu, search_menu):
         ac.current_url = path
         ac.setText(_('The location of the book'))
         copy_menu.addAction(ac)
+    elif dt == 'data-path':
+        path = data['loc']
+        ac = book_info.copy_link_action
+        path = get_gui().library_view.model().db.abspath(data['loc'], index_is_id=True)
+        if path:
+            path = os.path.join(path, DATA_DIR_NAME)
+            ac.current_url = path
+            ac.setText(_('The location of the book\'s data files'))
+            copy_menu.addAction(ac)
     else:
         field = data.get('field')
         if field is not None:
@@ -405,6 +508,7 @@ def add_item_specific_entries(menu, data, book_info, copy_menu, search_menu):
                 init_find_in_tag_browser(search_menu, find_action, field, value)
                 init_find_in_grouped_search(search_menu, field, value, book_info)
                 menu.addAction(init_manage_action(book_info.manage_action, field, value))
+                add_notes_context_menu_actions(menu, book_info, field, value)
             elif field == 'languages':
                 remove_value = langnames_to_langcodes((value,)).get(value, 'Unknown')
                 init_find_in_tag_browser(search_menu, find_action, field, value)
@@ -417,10 +521,17 @@ def add_item_specific_entries(menu, data, book_info, copy_menu, search_menu):
             ac.data = (field, remove_value, book_id)
             ac.setText(_('Remove %s from this book') % escape_for_menu(remove_name or data.get('original_value') or value))
             menu.addAction(ac)
+            # See if we need to add a click associated link menu line
+            link_map = get_gui().current_db.new_api.get_all_link_maps_for_book(data.get('book_id', -1))
+            link = link_map.get(field, {}).get(value)
+            if link:
+                menu.addAction(QIcon.ic('external-link'), _('Open associated link'),
+                               lambda : book_info.link_clicked.emit(link))
         else:
             v = data.get('original_value') or data.get('value')
-            copy_menu.addAction(QIcon.ic('edit-copy.png'), _('The text: {}').format(v),
-                                    lambda: QApplication.instance().clipboard().setText(v))
+            if v:
+                copy_menu.addAction(QIcon.ic('edit-copy.png'), _('The text: {}').format(v),
+                                        lambda: QApplication.instance().clipboard().setText(v))
     return search_internet_added
 
 
@@ -433,26 +544,67 @@ def create_copy_links(menu, data=None):
     library_id = '_hex_-' + library_id.encode('utf-8').hex()
     book_id = get_gui().library_view.current_id
 
-    def link(text, url):
+    def copy_to_clipboard_action(menu_text, value_text, before_action=None):
         def doit():
-            QApplication.instance().clipboard().setText(url)
-        menu.addAction(QIcon.ic('edit-copy.png'), text, doit)
+            QApplication.instance().clipboard().setText(value_text)
+        if before_action is not None:
+            action = QWidget(menu).addAction(QIcon.ic('edit-copy.png'), menu_text, doit)
+            menu.insertAction(before_action, action)
+        else:
+            menu.addAction(QIcon.ic('edit-copy.png'), menu_text, doit)
 
-    menu.addSeparator()
-    link(_('Link to show book in calibre'), f'calibre://show-book/{library_id}/{book_id}')
+    all_links = []
+    def link_action(text, url):
+        nonlocal all_links
+        all_links.append(url)
+        copy_to_clipboard_action(text, url)
+
+    sep = menu.addSeparator() # Note: separators are really actions
+
+    link_action(_('Link to show book in calibre'), f'calibre://show-book/{library_id}/{book_id}')
+    link_action(_('Link to show book details in a popup window'),
+                f'calibre://book-details/{library_id}/{book_id}')
+    mi = db.new_api.get_proxy_metadata(book_id)
+    if mi and mi.path:
+        with suppress(Exception):
+            data_files = db.new_api.list_extra_files(book_id, use_cache=True, pattern=DATA_FILE_PATTERN)
+            if data_files:
+                data_path = os.path.join(db.backend.library_path, mi.path, DATA_DIR_NAME)
+                link_action(_("Link to open book's data files folder"),
+                            bytes(QUrl.fromLocalFile(data_path).toEncoded()).decode('utf-8'))
     if data:
-        field = data.get('field')
-        if data['type'] == 'author':
-            field = 'authors'
-        if field and field in ('tags', 'series', 'publisher', 'authors') or is_category(field):
-            name = data['name' if data['type'] == 'author' else 'value']
-            eq = f'{field}:"={name}"'.encode().hex()
-            link(_('Link to show books matching {} in calibre').format(name),
-                 f'calibre://search/{library_id}?eq={eq}')
+        if data.get('kind', '') == 'notes':
+            field = data['field']
+            item_id = data['item_id']
+            note_data = db.notes_data_for(field, item_id)
+            if note_data is not None:
+                copy_to_clipboard_action(_('HTML for note'), note_data['doc'], sep)
+                copy_to_clipboard_action(_('Text for note'),
+                                         note_data['searchable_text'].partition('\n')[2], sep)
+            if field.startswith('#'):
+                field = '_' + field[1:]
+            url = f"calibre://show-note/{library_id}/{field}/id_{item_id}"
+            link_action(_('Link to show note in calibre'), url)
+        else:
+            field = data.get('field')
+            if data['type'] == 'author':
+                field = 'authors'
+            if field and field in ('tags', 'series', 'publisher', 'authors') or is_category(field):
+                name = data['name' if data['type'] == 'author' else 'value']
+                eq = f'{field}:"={name}"'.encode().hex()
+                link_action(_('Link to show books matching {} in calibre').format(name),
+                     f'calibre://search/{library_id}?eq={eq}')
 
     for fmt in db.formats(book_id):
         fmt = fmt.upper()
-        link(_('Link to view {} format of book').format(fmt.upper()), f'calibre://view-book/{library_id}/{book_id}/{fmt}')
+        link_action(_('Link to view {} format of book').format(fmt.upper()),
+                    f'calibre://view-book/{library_id}/{book_id}/{fmt}')
+
+    if all_links:
+        menu.addSeparator()
+        all_links.insert(0, '')
+        all_links.insert(0, mi.get('title') + ' - ' + ' & '.join(mi.get('authors')))
+        link_action(_('Copy all the above links'), '\n'.join(all_links))
 
 
 def details_context_menu_event(view, ev, book_info, add_popup_action=False, edit_metadata=None):
@@ -468,17 +620,31 @@ def details_context_menu_event(view, ev, book_info, add_popup_action=False, edit
     search_menu = QMenu(_('Search'), menu)
     search_menu.setIcon(QIcon.ic('search.png'))
     reindex_fmt_added = False
-    if url and url.startswith('action:'):
-        data = json_loads(from_hex_bytes(url.split(':', 1)[1]))
-        search_internet_added = add_item_specific_entries(menu, data, book_info, copy_menu, search_menu)
-        create_copy_links(copy_menu, data)
-        copy_links_added = True
-        reindex_fmt_added = 'reindex_fmt_added' in data
-    elif url and not url.startswith('#'):
-        ac = book_info.copy_link_action
-        ac.current_url = url
-        ac.setText(_('Copy link location'))
-        menu.addAction(ac)
+    if url:
+        def get_data():
+            kind, _, rest = url.partition(':')
+            data = json_loads(from_hex_bytes(rest))
+            data['kind'] = kind
+            return data
+
+        if url.startswith('action:'):
+            data = get_data()
+            search_internet_added = add_item_specific_entries(menu, data, book_info, copy_menu, search_menu)
+            create_copy_links(copy_menu, data)
+            copy_links_added = True
+            reindex_fmt_added = 'reindex_fmt_added' in data
+        elif url.startswith('notes:'):
+            data = get_data()
+            create_copy_links(copy_menu, data)
+            copy_links_added = True
+            search_internet_added = True
+            add_notes_context_menu_actions(menu, view, data['field'], data['value'])
+        elif not url.startswith('#'):
+            ac = book_info.copy_link_action
+            ac.current_url = url
+            ac.setText(_('Copy link location'))
+            menu.addAction(ac)
+            menu.addAction(QIcon.ic('external-link'), _('Open associated link'), lambda : book_info.link_clicked.emit(url))
     if not copy_links_added:
         create_copy_links(copy_menu)
 
@@ -499,11 +665,13 @@ def details_context_menu_event(view, ev, book_info, add_popup_action=False, edit
     menu.addSeparator()
     from calibre.gui2.ui import get_gui
     if add_popup_action:
-        ema = get_gui().iactions['Show Book Details'].menuless_qaction
-        menu.addAction(_('Open the Book details window') + '\t' + ema.shortcut().toString(QKeySequence.SequenceFormat.NativeText), book_info.show_book_info)
+        menu.addMenu(get_gui().iactions['Show Book Details'].qaction.menu())
     else:
-        ema = get_gui().iactions['Edit Metadata'].menuless_qaction
-        menu.addAction(_('Open the Edit metadata window') + '\t' + ema.shortcut().toString(QKeySequence.SequenceFormat.NativeText), edit_metadata)
+        # We can't open edit metadata from a locked window because EM expects to
+        # be editing the current book, which this book probably isn't
+        if edit_metadata is not None:
+            ema = get_gui().iactions['Edit Metadata'].menuless_qaction
+            menu.addAction(_('Open the Edit metadata window') + '\t' + ema.shortcut().toString(QKeySequence.SequenceFormat.NativeText), edit_metadata)
     if not reindex_fmt_added:
         menu.addSeparator()
         menu.addAction(_(
@@ -815,9 +983,10 @@ class BookInfo(HTMLDisplay):
     edit_book = pyqtSignal(int, object)
     edit_identifiers = pyqtSignal()
     find_in_tag_browser = pyqtSignal(object, object)
+    notes_resource_scheme = RESOURCE_URL_SCHEME
 
     def __init__(self, vertical, parent=None):
-        HTMLDisplay.__init__(self, parent)
+        HTMLDisplay.__init__(self, parent=parent, save_resources_in_document=False)
         self.vertical = vertical
         self.anchor_clicked.connect(self.link_activated)
         for x, icon in [
@@ -903,6 +1072,9 @@ class BookInfo(HTMLDisplay):
         html = render_html(mi, self.vertical, self.parent())
         set_html(mi, html, self)
 
+    def process_external_css(self, css):
+        return resolve_colors(css)
+
     def mouseDoubleClickEvent(self, ev):
         v = self.viewport()
         if v.rect().contains(self.mapFromGlobal(ev.globalPos())):
@@ -942,7 +1114,6 @@ class DetailsLayout(QSplitter):  # {{{
         super().__init__(orientation, parent)
         self.vertical = vertical
         self._children = []
-        self.min_size = QSize(190, 200) if vertical else QSize(120, 120)
         self.setContentsMargins(0, 0, 0, 0)
         self.splitterMoved.connect(self.do_splitter_moved,
                                    type=Qt.ConnectionType.QueuedConnection)
@@ -962,7 +1133,7 @@ class DetailsLayout(QSplitter):  # {{{
         self.resize_timer.start()
 
     def minimumSize(self):
-        return QSize(self.min_size)
+        return QSize(190, 200) if self.vertical else QSize(120, 120)
 
     def addWidget(self, child):
         if len(self._children) > 2:
@@ -973,7 +1144,7 @@ class DetailsLayout(QSplitter):  # {{{
         return len(self._children)
 
     def sizeHint(self):
-        return QSize(self.min_size)
+        return self.minimumSize()
 
     def restore_splitter_state(self):
         s = gprefs.get('book_details_widget_splitter_state')
@@ -984,9 +1155,9 @@ class DetailsLayout(QSplitter):  # {{{
             self.restoreState(s)
         self.setOrientation(Qt.Orientation.Vertical if self.vertical else Qt.Orientation.Horizontal)
 
-    def setGeometry(self, r):
-        super().setGeometry(r)
-        self.do_layout(r)
+    def setGeometry(self, *a):
+        super().setGeometry(*a)
+        self.do_layout(self.geometry())
 
     def do_splitter_moved(self, *args):
         gprefs['book_details_widget_splitter_state'] = bytearray(self.saveState())
@@ -1047,6 +1218,7 @@ class BookDetails(DetailsLayout):  # {{{
 
     show_book_info = pyqtSignal()
     open_containing_folder = pyqtSignal(int)
+    open_data_folder = pyqtSignal(int)
     view_specific_format = pyqtSignal(int, object)
     search_requested = pyqtSignal(object, object)
     remove_specific_format = pyqtSignal(int, object)
@@ -1115,7 +1287,7 @@ class BookDetails(DetailsLayout):  # {{{
     # }}}
 
     def __init__(self, vertical, parent=None):
-        DetailsLayout.__init__(self, vertical, parent)
+        super().__init__(vertical, parent)
         self.last_data = {}
         self.setAcceptDrops(True)
         self._layout = self
@@ -1155,8 +1327,11 @@ class BookDetails(DetailsLayout):  # {{{
                 url = url_for_author_search(data.where, author=data.author)
             safe_open_url(url)
 
-    def handle_click(self, link):
+    def handle_click_from_popup(self, link, parent=None):
+        parent = parent or self
         typ, val = link.partition(':')[::2]
+        from calibre.gui2.ui import get_gui
+        db = get_gui().current_db.new_api
 
         def search_term(field, val):
             append = ''
@@ -1171,8 +1346,14 @@ class BookDetails(DetailsLayout):  # {{{
             )
 
         def browse(url):
+            if url.startswith(DATA_DIR_NAME + '/') and self.last_data.get('book_id'):
+                base = get_gui().current_db.abspath(self.last_data['book_id'], index_is_id=True)
+                path = os.path.join(base, url)
+                qurl = QUrl.fromLocalFile(path)
+            else:
+                qurl = QUrl(url, QUrl.ParsingMode.TolerantMode)
             try:
-                safe_open_url(QUrl(url, QUrl.ParsingMode.TolerantMode))
+                safe_open_url(qurl)
             except Exception:
                 import traceback
                 traceback.print_exc()
@@ -1181,6 +1362,7 @@ class BookDetails(DetailsLayout):  # {{{
             data = json_loads(from_hex_bytes(val))
             dt = data['type']
             if dt == 'search':
+                field = data.get('field')
                 search_term(data['term'], data['value'])
             elif dt == 'author':
                 url = data['url']
@@ -1196,18 +1378,37 @@ class BookDetails(DetailsLayout):  # {{{
                     browse(data['url'])
             elif dt == 'path':
                 self.open_containing_folder.emit(int(data['loc']))
+            elif dt == 'data-path':
+                self.open_data_folder.emit(int(data['loc']))
             elif dt == 'devpath':
                 self.view_device_book.emit(data['loc'])
+        elif typ == 'notes':
+            data = json_loads(from_hex_bytes(val))
+            field = data.get('field')
+            # It shouldn't be possible for the field to be invalid or the
+            # note not to exist, but ...
+            if field and db.field_supports_notes(field):
+                item_id = data['item_id']
+                if item_id is not None and db.notes_for(field, item_id):
+                    return self.show_notes(field, item_id, parent)
         else:
             browse(link)
 
+    def handle_click(self, link):
+        self.handle_click_from_popup(link)
+
+    def show_notes(self, field, item_id, parent=None):
+        from calibre.gui2.dialogs.show_category_note import ShowNoteDialog
+        from calibre.gui2.ui import get_gui
+        ShowNoteDialog(field, item_id, get_gui().current_db.new_api, parent=parent or self).show()
+
     def mouseDoubleClickEvent(self, ev):
         ev.accept()
         self.show_book_info.emit()
 
     def show_data(self, data):
         try:
-            self.last_data = {'title':data.title, 'authors':data.authors}
+            self.last_data = {'title':data.title, 'authors':data.authors, 'book_id': getattr(data, 'id', None)}
         except Exception:
             self.last_data = {}
         self.book_info.show_data(data)
