@@ -5,7 +5,6 @@ __license__   = 'GPL v3'
 __copyright__ = '2011, Kovid Goyal <kovid@kovidgoyal.net>'
 __docformat__ = 'restructuredtext en'
 
-import errno
 import os
 from datetime import datetime
 from functools import partial
@@ -26,7 +25,7 @@ from calibre.gui2.metadata.basic_widgets import (
     AuthorsEdit, AuthorSortEdit, BuddyLabel, CommentsEdit, Cover, DateEdit,
     FormatsManager, IdentifiersEdit, LanguagesEdit, PubdateEdit, PublisherEdit,
     RatingEdit, RightClickButton, SeriesEdit, SeriesIndexEdit, TagsEdit, TitleEdit,
-    TitleSortEdit, show_locked_file_error,
+    TitleSortEdit
 )
 from calibre.gui2.metadata.single_download import FullFetch
 from calibre.gui2.widgets2 import CenteredToolButton
@@ -93,6 +92,12 @@ class MetadataSingleDialogBase(QDialog):
         self.prev_button = QPushButton(QIcon.ic('back.png'), _('Previous'),
                 self)
         self.prev_button.setShortcut(QKeySequence('Alt+Left'))
+        from calibre.gui2.actions.edit_metadata import DATA_FILES_ICON_NAME
+        self.data_files_button = QPushButton(QIcon.ic(DATA_FILES_ICON_NAME), _('Data files'), self)
+        self.data_files_button.setShortcut(QKeySequence('Alt+Space'))
+        self.data_files_button.setToolTip(_('Manage the extra data files associated with this book [{}]').format(
+            self.data_files_button.shortcut().toString(QKeySequence.SequenceFormat.NativeText)))
+        self.data_files_button.clicked.connect(self.manage_data_files)
 
         self.button_box.addButton(self.prev_button, QDialogButtonBox.ButtonRole.ActionRole)
         self.button_box.addButton(self.next_button, QDialogButtonBox.ButtonRole.ActionRole)
@@ -107,6 +112,7 @@ class MetadataSingleDialogBase(QDialog):
         self.l.addWidget(self.central_widget)
         ll = self.button_box_layout = QHBoxLayout()
         self.l.addLayout(ll)
+        ll.addWidget(self.data_files_button)
         ll.addSpacing(10)
         ll.addWidget(self.button_box)
 
@@ -191,14 +197,18 @@ class MetadataSingleDialogBase(QDialog):
         self.manage_authors_button = QToolButton(self)
         self.manage_authors_button.setIcon(QIcon.ic('user_profile.png'))
         self.manage_authors_button.setToolTip('<p>' + _(
-            'Manage authors. Use to rename authors and correct '
+            'Open the Manage Authors editor. Use to rename authors and correct '
             'individual author\'s sort values') + '</p>')
         self.manage_authors_button.clicked.connect(self.authors.manage_authors)
 
+        self.series_editor_button = QToolButton(self)
+        self.series_editor_button.setToolTip(_('Open the Manage Series editor'))
+        self.series_editor_button.setIcon(QIcon.ic('chapters.png'))
+        self.series_editor_button.clicked.connect(self.series_editor)
         self.series = SeriesEdit(self)
+        self.series.editor_requested.connect(self.series_editor)
         self.clear_series_button = QToolButton(self)
-        self.clear_series_button.setToolTip(
-               _('Clear series'))
+        self.clear_series_button.setToolTip(_('Clear series'))
         self.clear_series_button.clicked.connect(self.series.clear)
         self.series_index = SeriesIndexEdit(self, self.series)
         self.basic_metadata_widgets.extend([self.series, self.series_index])
@@ -229,7 +239,7 @@ class MetadataSingleDialogBase(QDialog):
 
         self.tags = TagsEdit(self)
         self.tags_editor_button = QToolButton(self)
-        self.tags_editor_button.setToolTip(_('Open Tag editor'))
+        self.tags_editor_button.setToolTip(_('Open the Tag editor. If Ctrl or Shift is pressed, open the Manage Tags editor'))
         self.tags_editor_button.setIcon(QIcon.ic('chapters.png'))
         self.tags_editor_button.clicked.connect(self.tags_editor)
         self.tags.tag_editor_requested.connect(self.tags_editor)
@@ -248,15 +258,20 @@ class MetadataSingleDialogBase(QDialog):
         self.paste_isbn_button = b = RightClickButton(self)
         b.setToolTip('<p>' +
                     _('Paste the contents of the clipboard into the '
-                      'identifiers prefixed with isbn: or url:. Or right click, '
-                      'to choose a different prefix.') + '</p>')
+                      'identifiers prefixed with an auto-detected prefix such as isbn: or url:. Or right click, '
+                      'and choose a specific prefix to use.') + '</p>')
         b.setIcon(QIcon.ic('edit-paste.png'))
         b.clicked.connect(self.identifiers.paste_identifier)
         b.setPopupMode(QToolButton.ToolButtonPopupMode.DelayedPopup)
         b.setMenu(QMenu(b))
         self.update_paste_identifiers_menu()
 
+        self.publisher_editor_button = QToolButton(self)
+        self.publisher_editor_button.setToolTip(_('Open the Manage Publishers editor'))
+        self.publisher_editor_button.setIcon(QIcon.ic('chapters.png'))
+        self.publisher_editor_button.clicked.connect(self.publisher_editor)
         self.publisher = PublisherEdit(self)
+        self.publisher.editor_requested.connect(self.publisher_editor)
         self.basic_metadata_widgets.append(self.publisher)
 
         self.timestamp = DateEdit(self)
@@ -375,6 +390,11 @@ class MetadataSingleDialogBase(QDialog):
     def data_changed(self):
         self.was_data_edited = True
 
+    def manage_data_files(self):
+        from calibre.gui2.dialogs.data_files_manager import DataFilesManager
+        d = DataFilesManager(self.db, self.book_id, self)
+        d.exec()
+
     def __call__(self, id_):
         self.book_id = id_
         self.books_to_refresh = set()
@@ -409,6 +429,12 @@ class MetadataSingleDialogBase(QDialog):
     def tags_editor(self, *args):
         self.tags.edit(self.db, self.book_id)
 
+    def publisher_editor(self, *args):
+        self.publisher.edit(self.db, self.book_id)
+
+    def series_editor(self, *args):
+        self.series.edit(self.db, self.book_id)
+
     def metadata_from_format(self, *args):
         mi, ext = self.formats_manager.get_selected_format_metadata(self.db,
                 self.book_id)
@@ -433,17 +459,9 @@ class MetadataSingleDialogBase(QDialog):
         if ext in ('pdf', 'cbz', 'cbr'):
             return self.choose_cover_from_pages(ext)
         try:
-            mi, ext = self.formats_manager.get_selected_format_metadata(self.db,
-                    self.book_id)
-        except OSError as err:
-            if getattr(err, 'errno', None) == errno.EACCES:  # Permission denied
-                import traceback
-                fname = err.filename if err.filename else 'file'
-                error_dialog(self, _('Permission denied'),
-                        _('Could not open %s. Is it being used by another'
-                        ' program?')%fname, det_msg=traceback.format_exc(),
-                        show=True)
-                return
+            mi, ext = self.formats_manager.get_selected_format_metadata(self.db, self.book_id)
+        except OSError as e:
+            e.locking_violation_msg = _('Could not read from book file.')
             raise
         if mi is None:
             return
@@ -593,18 +611,16 @@ class MetadataSingleDialogBase(QDialog):
             return True
         self.comments_edit_state_at_apply = {w:w.tab for w in self.comments_edit_state_at_apply}
         for widget in self.basic_metadata_widgets:
+            if hasattr(widget, 'validate_for_commit'):
+                title, msg, det_msg = widget.validate_for_commit()
+                if title is not None:
+                    error_dialog(self, title, msg, det_msg=det_msg, show=True)
+                    return False
             try:
-                if hasattr(widget, 'validate_for_commit'):
-                    title, msg, det_msg = widget.validate_for_commit()
-                    if title is not None:
-                        error_dialog(self, title, msg, det_msg=det_msg, show=True)
-                        return False
                 widget.commit(self.db, self.book_id)
                 self.books_to_refresh |= getattr(widget, 'books_to_refresh', set())
-            except OSError as err:
-                if getattr(err, 'errno', None) == errno.EACCES:  # Permission denied
-                    show_locked_file_error(self, err)
-                    return False
+            except OSError as e:
+                e.locking_violation_msg = _('Could not change on-disk location of this book\'s files.')
                 raise
         for widget in getattr(self, 'custom_metadata_widgets', []):
             self.books_to_refresh |= widget.commit(self.book_id)
@@ -791,7 +807,9 @@ class MetadataSingleDialog(MetadataSingleDialogBase):  # {{{
         sto(self.title_sort, self.manage_authors_button)
         sto(self.manage_authors_button, self.authors)
         create_row(1, self.authors, self.deduce_author_sort_button, self.author_sort)
-        sto(self.author_sort, self.series)
+        tl.addWidget(self.series_editor_button, 2, 0, 1, 1)
+        sto(self.author_sort, self.series_editor_button)
+        sto(self.series_editor_button, self.series)
         create_row(2, self.series, self.clear_series_button,
                 self.series_index, icon='trash.png')
 
@@ -851,8 +869,9 @@ class MetadataSingleDialog(MetadataSingleDialogBase):  # {{{
         create_row2(4, self.timestamp, self.timestamp.clear_button)
         sto(self.timestamp.clear_button, self.pubdate)
         create_row2(5, self.pubdate, self.pubdate.clear_button)
-        sto(self.pubdate.clear_button, self.publisher)
-        create_row2(6, self.publisher, self.publisher.clear_button)
+        sto(self.pubdate.clear_button, self.publisher_editor_button)
+        sto(self.publisher_editor_button, self.publisher)
+        create_row2(6, self.publisher, self.publisher.clear_button, front_button=self.publisher_editor_button)
         sto(self.publisher.clear_button, self.languages)
         create_row2(7, self.languages)
         self.tabs[0].spc_two = QSpacerItem(10, 10, QSizePolicy.Policy.Expanding,
@@ -957,8 +976,10 @@ class MetadataSingleDialogAlt1(MetadataSingleDialogBase):  # {{{
 
         tl.addWidget(self.swap_title_author_button, 0, 0, 2, 1)
         tl.addWidget(self.manage_authors_button, 2, 0, 1, 1)
-        tl.addWidget(self.paste_isbn_button, 12, 0, 1, 1)
+        tl.addWidget(self.series_editor_button, 6, 0, 1, 1)
         tl.addWidget(self.tags_editor_button, 6, 0, 1, 1)
+        tl.addWidget(self.publisher_editor_button, 9, 0, 1, 1)
+        tl.addWidget(self.paste_isbn_button, 12, 0, 1, 1)
 
         create_row(0, self.title, self.title_sort,
                    button=self.deduce_title_sort_button, span=2,
@@ -983,8 +1004,10 @@ class MetadataSingleDialogAlt1(MetadataSingleDialogBase):  # {{{
                    button=self.clear_identifiers_button, icon='trash.png')
         sto(self.clear_identifiers_button, self.swap_title_author_button)
         sto(self.swap_title_author_button, self.manage_authors_button)
-        sto(self.manage_authors_button, self.tags_editor_button)
-        sto(self.tags_editor_button, self.paste_isbn_button)
+        sto(self.manage_authors_button, self.series_editor_button)
+        sto(self.series_editor_button, self.tags_editor_button)
+        sto(self.tags_editor_button, self.publisher_editor_button)
+        sto(self.publisher_editor_button, self.paste_isbn_button)
         tl.addItem(QSpacerItem(1, 1, QSizePolicy.Policy.Fixed, QSizePolicy.Policy.Expanding),
                    13, 1, 1 ,1)
 
@@ -1111,8 +1134,10 @@ class MetadataSingleDialogAlt2(MetadataSingleDialogBase):  # {{{
 
         tl.addWidget(self.swap_title_author_button, 0, 0, 2, 1)
         tl.addWidget(self.manage_authors_button, 2, 0, 2, 1)
-        tl.addWidget(self.paste_isbn_button, 12, 0, 1, 1)
+        tl.addWidget(self.series_editor_button, 4, 0, 1, 1)
         tl.addWidget(self.tags_editor_button, 6, 0, 1, 1)
+        tl.addWidget(self.publisher_editor_button, 9, 0, 1, 1)
+        tl.addWidget(self.paste_isbn_button, 12, 0, 1, 1)
 
         create_row(0, self.title, self.title_sort,
                    button=self.deduce_title_sort_button, span=2,
@@ -1138,8 +1163,10 @@ class MetadataSingleDialogAlt2(MetadataSingleDialogBase):  # {{{
                    button=self.clear_identifiers_button, icon='trash.png')
         sto(self.clear_identifiers_button, self.swap_title_author_button)
         sto(self.swap_title_author_button, self.manage_authors_button)
-        sto(self.manage_authors_button, self.tags_editor_button)
-        sto(self.tags_editor_button, self.paste_isbn_button)
+        sto(self.manage_authors_button, self.series_editor_button)
+        sto(self.series_editor_button, self.tags_editor_button)
+        sto(self.tags_editor_button, self.publisher_editor_button)
+        sto(self.publisher_editor_button, self.paste_isbn_button)
         tl.addItem(QSpacerItem(1, 1, QSizePolicy.Policy.Fixed, QSizePolicy.Policy.Expanding),
                    13, 1, 1 ,1)
 
