@@ -1,5 +1,4 @@
 #!/usr/bin/env python
-# vim:fileencoding=utf-8
 # License: GPLv3 Copyright: 2015-2019, Kovid Goyal <kovid at kovidgoyal.net>
 
 
@@ -9,11 +8,11 @@ import shutil
 import subprocess
 import sys
 import tempfile
+from contextlib import suppress
 from io import BytesIO
-# We use explicit module imports so tracebacks when importing are more useful
-from PyQt5.QtCore import QBuffer, QByteArray, Qt
-from PyQt5.QtGui import (
-    QColor, QImage, QImageReader, QImageWriter, QPixmap, QTransform
+from qt.core import (
+    QBuffer, QByteArray, QColor, QImage, QImageReader, QImageWriter, QIODevice, QPixmap,
+    Qt, QTransform,
 )
 from threading import Thread
 
@@ -23,8 +22,9 @@ from calibre.ptempfile import TemporaryDirectory
 from calibre.utils.config_base import tweaks
 from calibre.utils.filenames import atomic_rename
 from calibre.utils.imghdr import what
+from calibre.utils.resources import get_image_path as I
 from calibre_extensions import imageops
-from polyglot.builtins import string_or_bytes, unicode_type
+from polyglot.builtins import string_or_bytes
 
 
 # Utilities {{{
@@ -53,11 +53,11 @@ def load_jxr_data(data):
     with TemporaryDirectory() as tdir:
         if isinstance(tdir, bytes):
             tdir = os.fsdecode(tdir)
-        with lopen(os.path.join(tdir, 'input.jxr'), 'wb') as f:
+        with open(os.path.join(tdir, 'input.jxr'), 'wb') as f:
             f.write(data)
         cmd = [get_exe_path('JxrDecApp'), '-i', 'input.jxr', '-o', 'output.tif']
         creationflags = subprocess.DETACHED_PROCESS if iswindows else 0
-        subprocess.Popen(cmd, cwd=tdir, stdout=lopen(os.devnull, 'wb'), stderr=subprocess.STDOUT, creationflags=creationflags).wait()
+        subprocess.Popen(cmd, cwd=tdir, stdout=open(os.devnull, 'wb'), stderr=subprocess.STDOUT, creationflags=creationflags).wait()
         i = QImage()
         if not i.load(os.path.join(tdir, 'output.tif')):
             raise NotImage('Failed to convert JPEG-XR image')
@@ -108,6 +108,14 @@ def gif_data_to_png_data(data, discard_animation=False):
 # Loading images {{{
 
 
+def set_image_allocation_limit(size_in_mb=1024):
+    with suppress(ImportError):  # for people running form source
+        from calibre_extensions.progress_indicator import (
+            set_image_allocation_limit as impl,
+        )
+        impl(size_in_mb)
+
+
 def null_image():
     ' Create an invalid image. For internal use. '
     return QImage()
@@ -117,24 +125,25 @@ def image_from_data(data):
     ' Create an image object from data, which should be a bytestring. '
     if isinstance(data, QImage):
         return data
+    set_image_allocation_limit()
     i = QImage()
     if not i.loadFromData(data):
         q = what(None, data)
         if q == 'jxr':
             return load_jxr_data(data)
-        raise NotImage('Not a valid image (detected type: {})'.format(q))
+        raise NotImage(f'Not a valid image (detected type: {q})')
     return i
 
 
 def image_from_path(path):
     ' Load an image from the specified path. '
-    with lopen(path, 'rb') as f:
+    with open(path, 'rb') as f:
         return image_from_data(f.read())
 
 
 def image_from_x(x):
     ' Create an image from a bytestring or a path or a file like object. '
-    if isinstance(x, unicode_type):
+    if isinstance(x, str):
         return image_from_path(x)
     if hasattr(x, 'read'):
         return image_from_data(x.read())
@@ -151,7 +160,7 @@ def image_and_format_from_data(data):
     ' Create an image object from the specified data which should be a bytestring and also return the format of the image '
     ba = QByteArray(data)
     buf = QBuffer(ba)
-    buf.open(QBuffer.ReadOnly)
+    buf.open(QIODevice.OpenModeFlag.ReadOnly)
     r = QImageReader(buf)
     fmt = bytes(r.format()).decode('utf-8')
     return r.read(), fmt
@@ -164,7 +173,8 @@ def image_to_data(img, compression_quality=95, fmt='JPEG', png_compression_level
     '''
     Serialize image to bytestring in the specified format.
 
-    :param compression_quality: is for JPEG and goes from 0 to 100. 100 being lowest compression, highest image quality
+    :param compression_quality: is for JPEG and WEBP and goes from 0 to 100.
+                                100 being lowest compression, highest image quality. For WEBP 100 means lossless with effort of 70.
     :param png_compression_level: is for PNG and goes from 0-9. 9 being highest compression.
     :param jpeg_optimized: Turns on the 'optimize' option for libjpeg which losslessly reduce file size
     :param jpeg_progressive: Turns on the 'progressive scan' option for libjpeg which allows JPEG images to be downloaded in streaming fashion
@@ -172,7 +182,7 @@ def image_to_data(img, compression_quality=95, fmt='JPEG', png_compression_level
     fmt = fmt.upper()
     ba = QByteArray()
     buf = QBuffer(ba)
-    buf.open(QBuffer.WriteOnly)
+    buf.open(QIODevice.OpenModeFlag.WriteOnly)
     if fmt == 'GIF':
         w = QImageWriter(buf, b'PNG')
         w.setQuality(90)
@@ -193,6 +203,8 @@ def image_to_data(img, compression_quality=95, fmt='JPEG', png_compression_level
     elif fmt == 'PNG':
         cl = min(9, max(0, png_compression_level))
         w.setQuality(10 * (9-cl))
+    elif fmt == 'WEBP':
+        w.setQuality(compression_quality)
     if not w.write(img):
         raise ValueError('Failed to export image as ' + fmt + ' with error: ' + w.errorString())
     return ba.data()
@@ -204,7 +216,7 @@ def save_image(img, path, **kw):
     `image_to_data()` function. '''
     fmt = path.rpartition('.')[-1]
     kw['fmt'] = kw.get('fmt', fmt)
-    with lopen(path, 'wb') as f:
+    with open(path, 'wb') as f:
         f.write(image_to_data(image_from_data(img), **kw))
 
 
@@ -215,7 +227,9 @@ def save_cover_data_to(
     compression_quality=90,
     minify_to=None,
     grayscale=False,
-    eink=False, letterbox=False,
+    eink=False,
+    letterbox=False,
+    letterbox_color='#000000',
     data_fmt='jpeg'
 ):
     '''
@@ -243,6 +257,8 @@ def save_cover_data_to(
         Works best with formats that actually support color indexing (i.e., PNG)
     :param letterbox: If True, in addition to fit resize_to inside minify_to,
         the image will be letterboxed (i.e., centered on a black background).
+    :param letterbox_color: If letterboxing is used, this is the background color
+        used. The default is black.
     '''
     fmt = normalize_format_name(data_fmt if path is None else os.path.splitext(path)[1][1:])
     if isinstance(data, QImage):
@@ -254,11 +270,11 @@ def save_cover_data_to(
         changed = fmt != orig_fmt
     if resize_to is not None:
         changed = True
-        img = img.scaled(resize_to[0], resize_to[1], Qt.IgnoreAspectRatio, Qt.SmoothTransformation)
+        img = img.scaled(int(resize_to[0]), int(resize_to[1]), Qt.AspectRatioMode.IgnoreAspectRatio, Qt.TransformationMode.SmoothTransformation)
     owidth, oheight = img.width(), img.height()
     nwidth, nheight = tweaks['maximum_cover_size'] if minify_to is None else minify_to
     if letterbox:
-        img = blend_on_canvas(img, nwidth, nheight, bgcolor='#000000')
+        img = blend_on_canvas(img, nwidth, nheight, bgcolor=letterbox_color)
         # Check if we were minified
         if oheight != nheight or owidth != nwidth:
             changed = True
@@ -266,7 +282,7 @@ def save_cover_data_to(
         scaled, nwidth, nheight = fit_image(owidth, oheight, nwidth, nheight)
         if scaled:
             changed = True
-            img = img.scaled(nwidth, nheight, Qt.IgnoreAspectRatio, Qt.SmoothTransformation)
+            img = img.scaled(int(nwidth), int(nheight), Qt.AspectRatioMode.IgnoreAspectRatio, Qt.TransformationMode.SmoothTransformation)
     if img.hasAlphaChannel():
         changed = True
         img = blend_image(img, bgcolor)
@@ -281,7 +297,7 @@ def save_cover_data_to(
         changed = True
     if path is None:
         return image_to_data(img, compression_quality, fmt, compression_quality // 10) if changed else data
-    with lopen(path, 'wb') as f:
+    with open(path, 'wb') as f:
         f.write(image_to_data(img, compression_quality, fmt, compression_quality // 10) if changed else data)
 # }}}
 
@@ -293,18 +309,18 @@ def blend_on_canvas(img, width, height, bgcolor='#ffffff'):
     w, h = img.width(), img.height()
     scaled, nw, nh = fit_image(w, h, width, height)
     if scaled:
-        img = img.scaled(nw, nh, Qt.IgnoreAspectRatio, Qt.SmoothTransformation)
+        img = img.scaled(int(nw), int(nh), Qt.AspectRatioMode.IgnoreAspectRatio, Qt.TransformationMode.SmoothTransformation)
         w, h = nw, nh
-    canvas = QImage(width, height, QImage.Format_RGB32)
+    canvas = QImage(int(width), int(height), QImage.Format.Format_RGB32)
     canvas.fill(QColor(bgcolor))
     overlay_image(img, canvas, (width - w)//2, (height - h)//2)
     return canvas
 
 
-class Canvas(object):
+class Canvas:
 
     def __init__(self, width, height, bgcolor='#ffffff'):
-        self.img = QImage(width, height, QImage.Format_RGB32)
+        self.img = QImage(int(width), int(height), QImage.Format.Format_RGB32)
         self.img.fill(QColor(bgcolor))
 
     def __enter__(self):
@@ -323,7 +339,7 @@ class Canvas(object):
 
 def create_canvas(width, height, bgcolor='#ffffff'):
     'Create a blank canvas of the specified size and color '
-    img = QImage(width, height, QImage.Format_RGB32)
+    img = QImage(int(width), int(height), QImage.Format.Format_RGB32)
     img.fill(QColor(bgcolor))
     return img
 
@@ -331,8 +347,8 @@ def create_canvas(width, height, bgcolor='#ffffff'):
 def overlay_image(img, canvas=None, left=0, top=0):
     ' Overlay the `img` onto the canvas at the specified position '
     if canvas is None:
-        canvas = QImage(img.size(), QImage.Format_RGB32)
-        canvas.fill(Qt.white)
+        canvas = QImage(img.size(), QImage.Format.Format_RGB32)
+        canvas.fill(Qt.GlobalColor.white)
     left, top = int(left), int(top)
     imageops.overlay(img, canvas, left, top)
     return canvas
@@ -347,7 +363,7 @@ def texture_image(canvas, texture):
 
 def blend_image(img, bgcolor='#ffffff'):
     ' Used to convert images that have semi-transparent pixels to opaque by blending with the specified color '
-    canvas = QImage(img.size(), QImage.Format_RGB32)
+    canvas = QImage(img.size(), QImage.Format.Format_RGB32)
     canvas.fill(QColor(bgcolor))
     overlay_image(img, canvas)
     return canvas
@@ -360,7 +376,7 @@ def add_borders_to_image(img, left=0, top=0, right=0, bottom=0, border_color='#f
     img = image_from_data(img)
     if not (left > 0 or right > 0 or top > 0 or bottom > 0):
         return img
-    canvas = QImage(img.width() + left + right, img.height() + top + bottom, QImage.Format_RGB32)
+    canvas = QImage(int(img.width() + left + right), int(img.height() + top + bottom), QImage.Format.Format_RGB32)
     canvas.fill(QColor(border_color))
     overlay_image(img, canvas, left, top)
     return canvas
@@ -373,7 +389,7 @@ def remove_borders_from_image(img, fuzz=None):
     absolute intensity units). Default is from a tweak whose default value is 10. '''
     fuzz = tweaks['cover_trim_fuzz_value'] if fuzz is None else fuzz
     img = image_from_data(img)
-    ans = imageops.remove_borders(img, max(0, fuzz))
+    ans = imageops.remove_borders(img, int(max(0, fuzz)))
     return ans if ans.size() != img.size() else img
 # }}}
 
@@ -381,7 +397,7 @@ def remove_borders_from_image(img, fuzz=None):
 
 
 def resize_image(img, width, height):
-    return img.scaled(int(width), int(height), Qt.IgnoreAspectRatio, Qt.SmoothTransformation)
+    return img.scaled(int(width), int(height), Qt.AspectRatioMode.IgnoreAspectRatio, Qt.TransformationMode.SmoothTransformation)
 
 
 def resize_to_fit(img, width, height):
@@ -408,10 +424,10 @@ def scale_image(data, width=60, height=80, compression_quality=70, as_png=False,
     if preserve_aspect_ratio:
         scaled, nwidth, nheight = fit_image(img.width(), img.height(), width, height)
         if scaled:
-            img = img.scaled(nwidth, nheight, Qt.KeepAspectRatio, Qt.SmoothTransformation)
+            img = img.scaled(int(nwidth), int(nheight), Qt.AspectRatioMode.KeepAspectRatio, Qt.TransformationMode.SmoothTransformation)
     else:
         if img.width() != width or img.height() != height:
-            img = img.scaled(width, height, Qt.IgnoreAspectRatio, Qt.SmoothTransformation)
+            img = img.scaled(int(width), int(height), Qt.AspectRatioMode.IgnoreAspectRatio, Qt.TransformationMode.SmoothTransformation)
     fmt = 'PNG' if as_png else 'JPEG'
     w, h = img.width(), img.height()
     return w, h, image_to_data(img, compression_quality=compression_quality, fmt=fmt)
@@ -429,7 +445,7 @@ def crop_image(img, x, y, width, height):
     img = image_from_data(img)
     width = min(width, img.width() - x)
     height = min(height, img.height() - y)
-    return img.copy(x, y, width, height)
+    return img.copy(int(x), int(y), int(width), int(height))
 
 # }}}
 
@@ -501,7 +517,7 @@ def quantize_image(img, max_colors=256, dither=True, palette=''):
         img = blend_image(img)
     if palette and isinstance(palette, string_or_bytes):
         palette = palette.split()
-    return imageops.quantize(img, max_colors, dither, [QColor(x).rgb() for x in palette])
+    return imageops.quantize(img, int(max_colors), dither, tuple(QColor(x).rgb() for x in palette))
 
 
 def eink_dither_image(img):
@@ -535,6 +551,7 @@ def run_optimizer(file_path, cmd, as_filter=False, input_data=None):
         else:
             os.close(fd)
         iname, oname = os.path.basename(file_path), os.path.basename(outfile)
+        input_size = os.path.getsize(file_path)
 
         def repl(q, r):
             cmd[cmd.index(q)] = r
@@ -561,27 +578,35 @@ def run_optimizer(file_path, cmd, as_filter=False, input_data=None):
             outw.start()
         raw = force_unicode(stderr.read())
         if p.wait() != 0:
+            p.stdout.close()
+            if as_filter:
+                p.stderr.close()
+                p.stdin.close()
             return raw
         else:
             if as_filter:
                 outw.join(60.0), inw.join(60.0)
+                p.stdin.close()
+                p.stderr.close()
+            p.stdout.close()
             try:
                 sz = os.path.getsize(outfile)
-            except EnvironmentError:
+            except OSError:
                 sz = 0
             if sz < 1:
                 return '%s returned a zero size image' % cmd[0]
-            shutil.copystat(file_path, outfile)
-            atomic_rename(outfile, file_path)
+            if sz < input_size:
+                shutil.copystat(file_path, outfile)
+                atomic_rename(outfile, file_path)
     finally:
         try:
             os.remove(outfile)
-        except EnvironmentError as err:
+        except OSError as err:
             if err.errno != errno.ENOENT:
                 raise
         try:
             os.remove(outfile + '.bak')  # optipng creates these files
-        except EnvironmentError as err:
+        except OSError as err:
             if err.errno != errno.ENOENT:
                 raise
 
@@ -595,24 +620,45 @@ def optimize_jpeg(file_path):
 def optimize_png(file_path, level=7):
     ' level goes from 1 to 7 with 7 being maximum compression '
     exe = get_exe_path('optipng')
-    cmd = [exe] + '-fix -clobber -strip all -o{} -out'.format(level).split() + [False, True]
+    cmd = [exe] + f'-fix -clobber -strip all -o{level} -out'.split() + [False, True]
     return run_optimizer(file_path, cmd)
 
 
+def run_cwebp(file_path, lossless, q, m, metadata):
+    exe = get_exe_path('cwebp')
+    q = max(0, min(q, 100))
+    m = max(0, min(m, 6))
+    cmd = [exe] + f'-mt -metadata {metadata} -q {q} -m {m} -o'.split() + [False, True]
+    if lossless:
+        cmd.insert(1, '-lossless')
+    return run_optimizer(file_path, cmd)
+
+
+def optimize_webp(file_path, q=100, m=6, metadata='all'):
+    ' metadata can be a comma seaprated list of all, none, exif, icc, xmp '
+    return run_cwebp(file_path, True, q, m, metadata)
+
+
 def encode_jpeg(file_path, quality=80):
     from calibre.utils.speedups import ReadOnlyFileBuffer
     quality = max(0, min(100, int(quality)))
     exe = get_exe_path('cjpeg')
-    cmd = [exe] + '-optimize -progressive -maxmemory 100M -quality'.split() + [unicode_type(quality)]
+    cmd = [exe] + '-optimize -progressive -maxmemory 100M -quality'.split() + [str(quality)]
     img = QImage()
     if not img.load(file_path):
         raise ValueError('%s is not a valid image file' % file_path)
     ba = QByteArray()
     buf = QBuffer(ba)
-    buf.open(QBuffer.WriteOnly)
+    buf.open(QIODevice.OpenModeFlag.WriteOnly)
     if not img.save(buf, 'PPM'):
         raise ValueError('Failed to export image to PPM')
-    return run_optimizer(file_path, cmd, as_filter=True, input_data=ReadOnlyFileBuffer(ba.data()))
+    data = ReadOnlyFileBuffer(ba.data())
+    buf.close()
+    return run_optimizer(file_path, cmd, as_filter=True, input_data=data)
+
+
+def encode_webp(file_path, quality=75, m=6, metadata='all'):
+    return run_cwebp(file_path, False, quality, m, metadata)
 # }}}
 
 
@@ -636,6 +682,10 @@ def test():  # {{{
             raise SystemExit('optimize_png failed: %s' % ret)
         if glob('*.bak'):
             raise SystemExit('Spurious .bak files left behind')
+        save_image(img, 'test.webp',  compression_quality=100)
+        ret = optimize_webp('test.webp')
+        if ret is not None:
+            raise SystemExit('optimize_webp failed: %s' % ret)
     quantize_image(img)
     oil_paint_image(img)
     gaussian_sharpen_image(img)
@@ -643,9 +693,11 @@ def test():  # {{{
     despeckle_image(img)
     remove_borders_from_image(img)
     image_to_data(img, fmt='GIF')
-    raw = subprocess.Popen([get_exe_path('JxrDecApp'), '-h'],
+    p = subprocess.Popen([get_exe_path('JxrDecApp'), '-h'],
                            creationflags=subprocess.DETACHED_PROCESS if iswindows else 0,
-                           stdout=subprocess.PIPE).stdout.read()
+                           stdout=subprocess.PIPE)
+    raw, _ = p.communicate()
+    p.wait()
     if b'JPEG XR Decoder Utility' not in raw:
         raise SystemExit('Failed to run JxrDecApp')
 # }}}
@@ -654,7 +706,7 @@ def test():  # {{{
 if __name__ == '__main__':  # {{{
     args = sys.argv[1:]
     infile = args.pop(0)
-    img = image_from_data(lopen(infile, 'rb').read())
+    img = image_from_data(open(infile, 'rb').read())
     func = globals()[args[0]]
     kw = {}
     args.pop(0)
@@ -679,6 +731,6 @@ if __name__ == '__main__':  # {{{
         bn = os.path.basename(infile)
         outf = bn.rpartition('.')[0] + '.' + '-output' + bn.rpartition('.')[-1]
     img = func(img, **kw)
-    with lopen(outf, 'wb') as f:
+    with open(outf, 'wb') as f:
         f.write(image_to_data(img, fmt=outf.rpartition('.')[-1]))
 # }}}
